@@ -31,6 +31,19 @@ def _callable(obj):
         return obj
 
 
+def get_value(d, name, multiple, method='get'):
+    func = getattr(d, method)
+    val = func(name)
+    if multiple:
+        if hasattr(d, 'getlist'):
+            return d.getlist(name)
+        elif hasattr(d, 'getall'):
+            return d.getall(name)
+        else:
+            return [val]
+    return val
+
+
 def noop(x):
     return x
 
@@ -48,24 +61,26 @@ class Arg(object):
     :param callable use: Callable used for converting or pre-processing the value.
         Defaults to noop.
         Example: ``use=lambda s: s.lower()``
+    :param bool multiple: Return a list of values for the argument. Useful for
+        querystrings or forms that pass multiple values to the same parameter,
+        e.g. ``/?name=foo&name=bar``
     :param str error: Custom error message to use if validation fails.
     """
     def __init__(self, type_=None, default=None, required=False,
-                 validate=None, use=None, error=None):
+                 validate=None, use=None, multiple=False, error=None):
         self.type = type_ or noop  # default to no type conversion
-        self.default = default
+        if multiple and default is None:
+            self.default = []
+        else:
+            self.default = default
         self.required = required
         self.validate = _callable(validate) or (lambda x: True)
         self.use = _callable(use) or noop
         self.error = error
+        self.multiple = multiple
 
-    def validated(self, value):
-        """Convert and validate the given value according to the ``type_``,
-        ``use``, and ``validate`` attributes.
-
-        :returns: The validated, converted value
-        :raises: ValidationError if validation fails
-        """
+    def _validate(self, value):
+        """Perform conversion and validation on ``value``."""
         ret = value
         # First convert the value
         try:
@@ -80,6 +95,20 @@ class Arg(object):
             raise ValidationError(self.error or msg)
         return ret
 
+    def validated(self, value):
+        """Convert and validate the given value according to the ``type_``,
+        ``use``, and ``validate`` attributes.
+
+        :returns: The validated, converted value
+        :raises: ValidationError if validation fails
+        """
+        print('in validated')
+        print(value)
+        if self.multiple and isinstance(value, list):
+            return [self._validate(each) for each in value]
+        else:
+            return self._validate(value)
+
 DEFAULT_TARGETS = ('querystring', 'form', 'json',)
 
 
@@ -91,6 +120,7 @@ class Parser(object):
     different targets, e.g. ``parse_json``, ``parse_querystring``, etc.
 
     :param tuple targets: Default targets to parse.
+    :param callable error_handler: Custom error handler function.
     """
 
     #: Maps target => method name
@@ -103,8 +133,9 @@ class Parser(object):
         'files': 'parse_files',
     }
 
-    def __init__(self, targets=DEFAULT_TARGETS):
+    def __init__(self, targets=DEFAULT_TARGETS, error_handler=None):
         self.targets = targets
+        self.error_handler = _callable(error_handler)
 
     def _validated_targets(self, targets):
         """Ensure that the given targets argument is valid.
@@ -133,19 +164,21 @@ class Parser(object):
         """
         value = None
         for target in self._validated_targets(targets or self.targets):
-            method_name = self.TARGET_MAP.get(target, None)
+            method_name = self.TARGET_MAP.get(target)
             if method_name:
                 method = getattr(self, method_name)
-                value = method(req, name)
+                value = method(req, name, argobj)
+            if argobj.multiple and not (isinstance(value, list) and len(value)):
+                continue
             # Found the value; validate and return it
             if value is not None:
                 return argobj.validated(value)
         if value is None:
-            if argobj.default:
+            if argobj.default is not None:
                 value = argobj.default
             else:
-                value = self.fallback(req, name)
-            if value is None and argobj.required:
+                value = self.fallback(req, name, argobj)
+            if not value and argobj.required:
                 raise ValidationError('Required parameter {0!r} not found.'.format(name))
         return value
 
@@ -166,7 +199,10 @@ class Parser(object):
                 for argname, argobj in iteritems(argmap)
             )
         except Exception as error:
-            self.handle_error(error)
+            if self.error_handler:
+                self.error_handler(error)
+            else:
+                self.handle_error(error)
 
     def use_args(self, argmap, req=None, targets=DEFAULT_TARGETS):
         """Decorator that injects parsed arguments into a view function or method.
@@ -191,37 +227,37 @@ class Parser(object):
 
     # Abstract Methods
 
-    def parse_json(self, req, name):
+    def parse_json(self, req, name, arg):
         """Pulls a JSON value from a request object or returns ``None`` if the
         value cannot be found.
         """
         return None
 
-    def parse_querystring(self, req, name):
+    def parse_querystring(self, req, name, arg):
         """Pulls a value from the query string of a request object or returns ``None`` if
         the value cannot be found.
         """
         return None
 
-    def parse_form(self, req, name):
+    def parse_form(self, req, name, arg):
         """Pulls a value from the form data of a request object or returns
         ``None`` if the value cannot be found.
         """
         return None
 
-    def parse_headers(self, req, name):
+    def parse_headers(self, req, name, arg):
         """Pulls a value from the headers or returns ``None`` if the value
         cannot be found.
         """
         return None
 
-    def parse_cookies(self, req, name):
+    def parse_cookies(self, req, name, arg):
         """Pulls a cookie value from the request or returns ``None`` if the value
         cannot be found.
         """
         return None
 
-    def parse_files(self, req, name):
+    def parse_files(self, req, name, arg):
         """Pull a file from the request or return ``None`` if the value file
         cannot be found.
         """
@@ -232,7 +268,7 @@ class Parser(object):
         """
         raise error
 
-    def fallback(self, req, name):
+    def fallback(self, req, name, arg):
         """Called if all other parsing functions (parse_json, parse_form...) return
         ``None``.
         """
